pub enum Token<S> {
Show 41 variants
Identifier(S),
PathField(S),
FunctionCall(S),
Operator(S),
StringLiteral(StringLiteralToken<S>),
RawStringLiteral(RawStringLiteralToken<S>),
IntegerLiteral(i64),
FloatLiteral(NotNan<f64>),
RegexLiteral(S),
TimestampLiteral(S),
ReservedIdentifier(S),
InvalidToken(char),
If,
Else,
Null,
False,
True,
Abort,
Return,
Colon,
Comma,
Dot,
LBrace,
LBracket,
LParen,
Newline,
RBrace,
RBracket,
RParen,
SemiColon,
Underscore,
Escape,
Arrow,
Ampersand,
Percent,
Equals,
MergeEquals,
Bang,
Question,
LQuery,
RQuery,
}Variants
Identifier(S)
PathField(S)
FunctionCall(S)
Operator(S)
StringLiteral(StringLiteralToken<S>)
RawStringLiteral(RawStringLiteralToken<S>)
IntegerLiteral(i64)
FloatLiteral(NotNan<f64>)
RegexLiteral(S)
TimestampLiteral(S)
ReservedIdentifier(S)
InvalidToken(char)
If
Else
Null
False
True
Abort
Return
Colon
Comma
Dot
LBrace
LBracket
LParen
Newline
RBrace
RBracket
RParen
SemiColon
Underscore
Escape
Arrow
Ampersand
Percent
Equals
MergeEquals
Bang
Question
LQuery
The {L,R}Query token is an “instruction” token. It does not represent any character in the source, instead it represents the start or end of a sequence of tokens that together form a “query”.
Some examples:
. => LQuery, Dot, RQuery
.foo => LQuery, Dot, Ident, RQuery
foo.bar[2] => LQuery, Ident, Dot, Ident, LBracket, Integer, RBracket, RQuery
foo().bar => LQuery, FunctionCall, LParen, RParen, Dot, Ident, RQuery
[1].foo => LQuery, LBracket, Integer, RBracket, Dot, Ident, RQuery
{ .. }[0] => LQuery, LBrace, ..., RBrace, LBracket, ... RBracket, RQueryThe final example shows how the lexer does not care about the semantic validity of a query (as in, getting the index from an object does not work), it only signals that one exists.
Some non-matching examples:
. foo => Dot, Identifier
foo() .a => FunctionCall, LParen, RParen, LQuery, Dot, Ident, RQuery
[1] [2] => RBracket, Integer, LBracket, RBracket, Integer, RBracketThe reason these tokens exist is to allow the parser to remain whitespace-agnostic, while still being able to distinguish between the above two groups of examples.
RQuery